// file      : xsd-frontend/semantic-graph/complex.hxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005-2006 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifndef XSD_FRONTEND_SEMANTIC_GRAPH_COMPLEX_HXX
#define XSD_FRONTEND_SEMANTIC_GRAPH_COMPLEX_HXX

#include <xsd-frontend/semantic-graph/elements.hxx>
#include <xsd-frontend/semantic-graph/compositors.hxx>

#include <cult/containers/vector.hxx>

namespace XSDFrontend
{
  namespace SemanticGraph
  {
    class Complex: public virtual Type, public virtual Scope
    {
      typedef
      Cult::Containers::Vector<Inherits*>
      InheritsList;

    public:
      typedef
      Bits::PointerIterator<InheritsList::ConstIterator>
      InheritsIterator;

      InheritsIterator
      inherits_begin () const
      {
        return inherits_.begin ();
      }

      InheritsIterator
      inherits_end () const
      {
        return inherits_.end ();
      }

      Boolean
      inherits_p () const
      {
        return inherits_begin () != inherits_end ();
      }

    public:
      Boolean
      contains_compositor_p ()
      {
        return contains_compositor_ != 0;
      }

      ContainsCompositor&
      contains_compositor ()
      {
        assert (contains_compositor_ != 0);
        return *contains_compositor_;
      }

    protected:
      friend class Bits::Graph<Node, Edge>;

      Complex (); // Virtual inheritance (Enumeration).
      Complex (Path const& file, UnsignedLong line, UnsignedLong column);

      using Type::add_edge_right;
      using Scope::add_edge_left;

      Void
      add_edge_left (Inherits& e)
      {
        inherits_.push_back (&e);
      }

      Void
      add_edge_left (ContainsCompositor& e)
      {
        assert (contains_compositor_ == 0);
        contains_compositor_ = &e;
      }

    private:
      InheritsList inherits_;
      ContainsCompositor* contains_compositor_;
    };
  }
}

#endif  // XSD_FRONTEND_SEMANTIC_GRAPH_COMPLEX_HXX
